#include <stm32f4xx.h>
#include <string.h>

#define BUFFER_SIZE 16

volatile char received_buffer[BUFFER_SIZE+1];
static uint8_t DataReceivedCounter = 0; 

void USART1_init(uint32_t baudrate);
void USARTCommandReceived(char * command);
void ClearCommand(void);
void SendData(USART_TypeDef* USARTx, volatile char *s);
void Start_PWM(uint32_t dutycycle);
void Stop_PWM(void);

// Funkcija za inicijalizaciju USART1 sucelja na GPIOB PB6 i PB7

void USART1_init(uint32_t baudrate)
{
	GPIO_InitTypeDef GPIO_InitStruct;
	USART_InitTypeDef USART_InitStruct; 
	NVIC_InitTypeDef NVIC_InitStruct; 
	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);
	
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
    
	GPIO_InitStruct.GPIO_Pin = GPIO_Pin_6 | GPIO_Pin_7; 
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF; 			
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;		
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;			
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;			
	GPIO_Init(GPIOB, &GPIO_InitStruct);	
    
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_USART1);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_USART1);

	USART_InitStruct.USART_BaudRate = baudrate;				
	USART_InitStruct.USART_WordLength = USART_WordLength_8b;
	USART_InitStruct.USART_StopBits = USART_StopBits_1;		
	USART_InitStruct.USART_Parity = USART_Parity_No;		
	USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None; 
	USART_InitStruct.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
	USART_Init(USART1, &USART_InitStruct);	
    
	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE); 
	
	NVIC_InitStruct.NVIC_IRQChannel = USART1_IRQn;		 
	NVIC_InitStruct.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStruct.NVIC_IRQChannelSubPriority = 0;		 
	NVIC_InitStruct.NVIC_IRQChannelCmd = ENABLE;			 
	NVIC_Init(&NVIC_InitStruct);							 

	USART_Cmd(USART1, ENABLE);
}

void USART2_init(uint32_t baudrate)
{
    GPIO_InitTypeDef GPIO_InitStruct;
	USART_InitTypeDef USART_InitStruct; 
    
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);
    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);
    
    GPIO_InitStruct.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6; 
	GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF; 			
	GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;		
	GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;			
	GPIO_InitStruct.GPIO_PuPd = GPIO_PuPd_UP;			
	GPIO_Init(GPIOD, &GPIO_InitStruct);	
	      
    GPIO_PinAFConfig(GPIOD, GPIO_PinSource5, GPIO_AF_USART2);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource6, GPIO_AF_USART2);
    
    USART_InitStruct.USART_BaudRate = baudrate;				
	USART_InitStruct.USART_WordLength = USART_WordLength_8b;
	USART_InitStruct.USART_StopBits = USART_StopBits_1;		
	USART_InitStruct.USART_Parity = USART_Parity_No;		
	USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None; 
	USART_InitStruct.USART_Mode = USART_Mode_Tx | USART_Mode_Rx;
	USART_Init(USART2, &USART_InitStruct);	
    
    USART_Cmd(USART2, ENABLE);
}

void USART1_IRQHandler(void)
{
	//check the type of interrupt to make sure we have received some data.
	if( USART_GetITStatus(USART1, USART_IT_RXNE) ){
		char t = USART1->DR; //Read the character that we have received

		if( (DataReceivedCounter < BUFFER_SIZE) && t != '\r' && t!='\n' ){
			received_buffer[DataReceivedCounter] = t;
			DataReceivedCounter++;
		}
		else{ // otherwise reset the character counter and print the received string
			DataReceivedCounter = 0;
			//only raise a command event if the enter key was pressed otherwise just clear
			if(t == '\r'){
				USARTCommandReceived(received_buffer);
			}
			ClearCommand();
		}
	}
}

void USARTCommandReceived(char* command)
{
    SendData(USART2, command);
    SendData(USART2, " ");
    if(strcmp(command, "PUMP_ON") == 0)
		{
      GPIO_SetBits(GPIOD, GPIO_Pin_15);
			Start_PWM(100);			
    }
		else if(strcmp(command, "PUMP_OFF") == 0)
    {
			GPIO_ResetBits(GPIOD, GPIO_Pin_15);
			Stop_PWM();
		}
}


void ClearCommand(void)
{
	int i =0;
	for(i=0;i < BUFFER_SIZE; i++)
    {
		received_buffer[i] = 0;
	}
}

void SendData(USART_TypeDef* USARTx, volatile char *s){

	while(*s){
		// wait until data register is empty
		while( !(USARTx->SR & 0x00000040) );
		USART_SendData(USARTx, *s);
		*s++;
	}
}

